!
! lanczostransfloquet --- carry out a dressed Rydberg calculation using
!       the band-Lanczos algorithm to treat the Floquet matrix
! Christian Buth
! 7 Nov 2006
!


      subroutine lanczostransfloquet(nmat, nmain, niter, ceig, cpolest, nvec, nl,  &
     &          nphmin, lmin, coredip, normsq, jnd)
      use floquetmat
      use sparse
      use lanczos
      implicit none

      integer, intent(in) :: nmat, nmain, niter, nvec, nl, nphmin, lmin
      integer, intent(out) :: jnd
      complex(dblprec), intent(in) :: coredip(nmat)
      complex(dblprec), intent(out) :: normsq, ceig(niter), cpolest(niter)

      integer i, j, l, mu, neigenv, ncartsv, iparam(10), icartsv(1), evlist(1)
      real(dblprec) thresh(3)
      complex(dblprec), allocatable :: lanczosvectors(:,:,:), &
     &          Tmat(:,:), e2(:), z(:,:), zev(:,:), ev(:,:)
      complex(dblprec) zp
      external matrixvector

      write (*,*) 'Diagonalizing the complex symmetric Hamiltonian ', &
     &            'with dimension =', nmat
      write (*,*) 'Number of band-Lanczos iterations ', niter
      write (*,*) 'Number of Lanczos vectors ', nmain

      ! Error checking
      if (niter > nmat) then
        write (*,*) 'Warning: the number of band-Lanczos iterations is larger ',  &
     &              'than the matrix dimension!'
      end if

      if (nmain /= 1) then
        stop 'lanczostransfloquet: nmain /= 1'
      end if

      ! Allocate memory
      neigenv = 1               ! For the memory allocation to work
      allocate(lanczosvectors(nmat, nmain, 3), Tmat(nmain+1, niter),  &
     &          e2(niter), z(2*nmain, niter), zev(2*nmain, niter),    &
     &          ev(nmat,neigenv))

      ! Read zHmat from disc
      sparsematfile%readfile = .true.
      sparsematfile%fmtxcom  = 393
      sparsematfile%fmtxdia  = 394
      sparsematfile%fmtxoff  = 395
      call openmatrixfile(sparsematfile)

      l = 1
      mu = 0
      lanczosvectors = 0
      ! Generate a dipole matrix elements start vector
      do i=1, nvec
        j = nl * nvec * (mu - nphmin) + nvec * (l - lmin) + i
        lanczosvectors(j, nmain, 2) = coredip(i)
      end do
      ! Normalize the start vector, dot_product is a Hermitian product
      normsq = dot_product(conjg(lanczosvectors(:, nmain, 2)), lanczosvectors(:, nmain, 2))
      lanczosvectors(:, nmain, 2) = lanczosvectors(:, nmain, 2) / sqrt(normsq)

      ! Set parameters for complex symmetric band-Lanczos
      neigenv     = 0          ! Normal band-Lanczos run
      ncartsv     = 0
      iparam(1)   = 1
      iparam(2)   = niter
      iparam(3:6) = 0
      call zblanc(nmat, nmain, niter, neigenv, ncartsv, iparam, Tmat,  &
     &          lanczosvectors, z, ev, icartsv, matrixvector)

      ! Set parameters for zbnddia
      thresh(1) = 0.07          ! Minimal pole strength for output
      thresh(2) = 1
      thresh(3) = 0
      neigenv   = 0
      evlist(1) = 0
      iparam(1) = 0
      ceig(:)   = (0.0D0, 0.0D0)
      call zbnddia(niter, nmain, Tmat, ceig, e2, zev, thresh,  &
     &          neigenv, evlist, iparam, jnd)

      ! Determine the complex "polestrengths" in Riss-Meyer definition
      do j=1, jnd
        zp = (0.0D0, 0.0D0)
        do i=1, nmain
          zp = zp + zev(i,j)**2
        end do
        cpolest(j) = sqrt(zp)
      end do

      call closematrixfile(sparsematfile)
      deallocate(lanczosvectors, Tmat, e2, z, zev, ev)
      end subroutine lanczostransfloquet
