!
! readatom --- read file containing atomic matrix elements
!       as determined by oneel into main memory
! Robin Santra, Christian Buth
! 19 May 2006
!
!
! Read atomic one-electron eigenenergies (eval), dipole matrix elements
! (Datom), and matrix elements of the CAP operator W (Watom);
! these quantities have to be calculated, before running this code,
! using the finite-element-based one-electron code written
! by Kevin Christ and Mark Baertschy
!


      subroutine readatom(nl, lmin, lmax, nvec, nvlnc, eval, SESdipatm,  &
     &          SESfTfatm,SESpotatm,SESWatom,li,init,Eground)
      use floquetmat, readatomdbl => readatom
      implicit none

      integer, intent(in) :: nl, lmin, lmax, nvec, li, init
      integer, intent(out) :: nvlnc
      real(dblprec), intent(out) :: eval(nvec,lmin:lmax),  &
     &          Eground
      complex(dblprec), intent(out) ::                                &
     &          SESWatom(nvec,nvec,lmin:lmax),                        &
     &          SESdipatm(nvec,lmin:lmax,nvec,lmin:lmax),             &
     &          SESfTfatm(nvec,nvec,lmin:lmax),                       &
     &          SESpotatm(nvec,nvec,lmin:lmax)

      integer auxnl, auxnvec, auxnode
      integer i, j, k, l

      open(unit=10, file='atom.dat', access='sequential', status='old',  &
     &     form='formatted')

 21   format(3i4)
 22   format(d22.15,i5)
 23   format(3d22.15)

      read(10,21) auxnl, auxnvec, nvlnc
      if (auxnl /= nl .or. auxnvec /= nvec .or. nvlnc < 1) then
        write(*,*) 'readatom: Inconsistency detected when ', &
     &             'reading atom.dat'
        write(*,*) 'auxnl /= nl .or. auxnvec /= nvec .or. nvlnc < 1'
        write(*,*) 'auxnl =', auxnl, ', nl =', nl, ', auxnvec =', &
     &             auxnvec, ', nvec =', nvec, ', nvlnc =',nvlnc
        write(*,*)
        close(unit=10)
        stop 'readatom: auxnl /= nl .or. auxnvec /= nvec .or. nvlnc < 1'
      end if
      do l = lmin, lmax
        do j = 1, nvec
          read(10,22) eval(j,l), auxnode
        end do
      end do
      read(10,23) ((((SESdipatm(i,j,k,l),i=1,nvec),j=lmin,lmax), &
     &     k=1,nvec),l=lmin,lmax)
      read(10,23) (((SESfTfatm(i,j,k),i=1,nvec),j=1,nvec),k=lmin,lmax)
      read(10,23) (((SESpotatm(i,j,k),i=1,nvec),j=1,nvec),k=lmin,lmax)
      read(10,23) (((SESWatom(i,j,k),i=1,nvec),j=1,nvec),k=lmin,lmax)
      close(unit=10)

      ! It is assumed that the initial valence state is the "init"
      ! one of angular momentum "li"
      Eground = eval(init,li)
      write(*,*) 'Energy of initial valence level: Eground = ', Eground, 'Hartree'
      end subroutine readatom
