!
! fTfmtrx --- Calculate matrix representation of the transformed
!       kinetic energy operator f^-1 T f^-1 for the smooth
!       exterior complex scaled Hamiltonian
! Christian Buth
! 22 May 2006
!


! #define fltprec quadprec
#define fltprec dblprec


      subroutine fTfmtrx(fTf, nstate, nl, egfxn, egder, ngrd, xh)
      use fortrantl
      implicit none

      integer, intent(in) :: nstate, nl, ngrd
      real(dblprec), intent(in) :: xh
      real(dblprec), intent(in) :: egfxn(ngrd,nstate,nl), egder(ngrd,nstate,nl)
      complex(dblprec), intent(out) :: fTf(nstate,nstate,nl)
!       complex(dblprec) tempfTf(nstate,nstate,nl), tempfTf2(nstate,nstate,nl)
!       integer l, k, j

      call anader_fTfmtrx(fTf, nstate, nl, egfxn, egder, ngrd, xh)
!       call numder_fTfmtrx(tempfTf, nstate, nl, egfxn, ngrd, xh)
!       tempfTf2 = tempfTf - fTf

!       do l = 1, nl
!         do k = 1, nstate
!           do j = 1, k
!             if (abs(tempfTf2(k,j,l)) > 1.0D-6) then
!               write (*,*) k, j, l, fTf(k,j,l), tempfTf(k,j,l)
!             end if
!           end do
!          end do
!       end do
      end subroutine fTfmtrx


!
! anader_fTfmtrx --- evaluate the transformed kinetic energy
!       operator for the SES CAP using analytical derivatives
!

      subroutine anader_fTfmtrx(fTf, nstate, nl, egfxn, egder, ngrd, xh)
      use quadrature
      use SESpath
      implicit none

      integer, intent(in) :: nstate, nl, ngrd
      real(dblprec), intent(in) :: xh
      real(dblprec), intent(in) :: egfxn(ngrd,nstate,nl), egder(ngrd,nstate,nl)
      complex(dblprec), intent(out) :: fTf(nstate,nstate,nl)
      integer i, j, k, l
      real(dblprec) pf
      real(dblprec) x, r, numinterr, suminterr, re1, im1, re2, im2
      real(dblprec), allocatable :: rework(:), imwork(:)
      complex(dblprec) z, zt

      numinterr = 0.0D0
      suminterr = 0.0D0
      allocate(rework(ngrd), imwork(ngrd))

      do l = 1, nl
         do k = 1, nstate
            do j = 1, k
               do i = 1, ngrd
                  x = xh * real(i, dblprec)
                  pf = -2.0D0 * x / 2.0D0      ! Integration
                                ! measure -hbar^2 / (2 m) in a.u. substitution
                                ! r = x^2 in the two derivatives

                  ! First term
                  z =   - cmplx(egder(i,j,l), kind=dblprec)
                  z = z * cmplx(egder(i,k,l), kind=dblprec)
                  z = z / pathf(x**2)**2
                  zt = z
                  ! Second term
                  z =     cmplx(egder(i,j,l), kind=dblprec)
                  z = z * cmplx(egfxn(i,k,l), kind=dblprec)
                  z = z / pathf(x**2)**3 * pathfp(x**2)
                  zt = zt + z
                  ! Third term
                  z =     cmplx(egfxn(i,j,l), kind=dblprec)
                  z = z * cmplx(egder(i,k,l), kind=dblprec)
                  z = z / pathf(x**2)**3 * pathfp(x**2)
                  zt = zt + z
                  ! Forth term
                  z =   - cmplx(egfxn(i,j,l), kind=dblprec)
                  z = z * cmplx(egfxn(i,k,l), kind=dblprec)
                  z = z / pathf(x**2)**4 * pathfp(x**2)**2
                  zt = zt + z

                  ! Unscaled kinetic energy
                  z =     cmplx(egder(i,j,l), kind=dblprec)
                  z = z * cmplx(egder(i,k,l), kind=dblprec)
                  zt = zt + z

                  ! Eliminate spurious contamination
                  zt = zt * cmplx(pf, kind=dblprec)
!                   if (x**2 < sesrlow) then
!                     zt = (0.0D0, 0.0D0)
!                   end if
!                   if (x**2 < sesrlow .and. abs(zt) > 1.0D-10) then
!                     write (*,*) j, k, l, x**2, abs(zt)
!                   end if
                  rework(i) = real(zt       , dblprec)
                  imwork(i) = real(aimag(zt), dblprec)

!                   ! Write out a selected integral
!                   if (l == 1 .and. j == 1 .and. k == 1) then
!                     if (x == xh) then
!                       write (*,*) 'Write out a selected integral'
!                     end if
!                     write (101,'(4(F10.6,A))') x, ' ', x**2, ' ', rework(i), ' ', imwork(i), ' '
!                   end if
!                   if (l == 1 .and. j == 1 .and. k == 200) then
!                     write (102,'(4(F10.6,A))') x, ' ', x**2, ' ', rework(i), ' ', imwork(i), ' '
!                   end if
!                   if (l == 1 .and. j == 100 .and. k == 100) then
!                     write (103,'(4(F10.6,A))') x, ' ', x**2, ' ', rework(i), ' ', imwork(i), ' '
!                   end if
               end do

               ! Numerical integration
!                re1 = midpointrule (ngrd,xh,rework)
!                im1 = midpointrule (ngrd,xh,imwork)
!                re1 = trapezoidrule(ngrd,xh,rework)
!                im1 = trapezoidrule(ngrd,xh,imwork)
               re1 = threeeighthsrule(ngrd,xh,rework)
               im1 = threeeighthsrule(ngrd,xh,imwork)
               re2 = simpsonsrule    (ngrd,xh,rework)
               im2 = simpsonsrule    (ngrd,xh,imwork)
               r = (re1 - re2)**2 + (im1 - im2)**2
               if (r > numinterr) then
                 numinterr = r
               end if
               suminterr = suminterr + sqrt(r)
               fTf(j,k,l) = cmplx(re2, im2, dblprec)
               fTf(k,j,l) = fTf(j,k,l)          ! fTf is complex symmetric
            end do
         end do
      end do

      write (*,*) '        Error in numerical integration (analytic)', sqrt(numinterr),  &
     &          '(maximum), ', suminterr / real(nl * nstate * (nstate + 1) / 2,  &
     &          dblprec), '(mean)'
      deallocate(rework, imwork)
      end subroutine anader_fTfmtrx


! !
! ! numder_fTfmtrx --- evaluate the transformed kinetic energy
! !       operator for the SES CAP using numerical derivatives
! !
!
!       subroutine numder_fTfmtrx(fTf, nstate, nl, egfxn, ngrd, xh)
!       use quadrature
!       use SESpath
!       implicit none
!
!       integer, intent(in) :: nstate, nl, ngrd
!       real(dblprec), intent(in) :: xh
!       real(dblprec), intent(in) :: egfxn(ngrd,nstate,nl)
!       complex(dblprec), intent(out) :: fTf(nstate,nstate,nl)
!       integer i, j, k, l
!       real(fltprec) pf
!       real(dblprec) x, r, numinterr, suminterr, re1, im1, re2, im2
!       real(dblprec), allocatable :: rework(:), imwork(:)
!       complex(fltprec) z, zh, zt1, zt2
! !       real(dblprec) rint
! !       external rint
!
! !
! ! Numerical differentiation of the radial wavefunction
! !
! ! In an interval with the three-point formula
! ! (central difference formula)
! ! f'(x) = [f(x+h) - f(x-h)] / (2h)
! !
! ! At the left border of an interval with the forward difference
! ! f'(x) = [f(x+h) - f(x)] / h
! !
! ! At the right border of an interval with the backward difference
! ! f'(x) = [f(x) - f(x-h)] / h
! !
!
!       numinterr = 0.0D0
!       suminterr = 0.0D0
!       zh = cmplx(xh, kind=fltprec)
!       allocate(rework(ngrd), imwork(ngrd))
!
!       do l = 1, nl
!          do k = 1, nstate
!             do j = 1, k
!                ! Forward difference formula
!                i = 1
!                x = xh
!                pf = real(2.0D0 * x / 2.0D0  / (4.0D0 * x**2), fltprec)     ! Integration
!                                 ! measure hbar^2 / (2 m) in a.u. substitution
!                                 ! r = x^2 in derivative
!                ! Evaluate numerical derivative of the bra state vector
!                ! phi(x) / f(x)
!                z =     cmplx(egfxn(i+1,j,l), kind=fltprec) / cmplx(pathf((x+xh)**2), kind=fltprec)
!                z = z - cmplx(egfxn(i  ,j,l), kind=fltprec) / cmplx(pathf((x   )**2), kind=fltprec)
!                zt1 = z / zh
!                z =     cmplx(egfxn(i+1,j,l), kind=fltprec)
!                z = z - cmplx(egfxn(i  ,j,l), kind=fltprec)
!                zt2 = z / zh
!                ! Evaluate numerical derivative of the ket state vector
!                ! phi(x) / f(x)
!                z =     cmplx(egfxn(i+1,k,l), kind=fltprec) / cmplx(pathf((x+xh)**2), kind=fltprec)
!                z = z - cmplx(egfxn(i  ,k,l), kind=fltprec) / cmplx(pathf((x   )**2), kind=fltprec)
!                z = z / zh
!                zt1 = zt1 * z * cmplx(pf, kind=fltprec)
!                z =     cmplx(egfxn(i+1,k,l), kind=fltprec)
!                z = z - cmplx(egfxn(i  ,k,l), kind=fltprec)
!                z = z / zh
!                zt2 = zt2 * z * cmplx(pf, kind=fltprec)
!                zt1 = zt1 - zt2
!                rework(i) = real (zt1, dblprec)
!                imwork(i) = aimag(zt1)
!
!                ! Central difference formula
!                do i = 2, ngrd-1
!                   x = xh * real(i, dblprec)
!                   pf = real(2.0D0 * x / 2.0D0  / (4.0D0 * x**2), fltprec)     ! Integration
!                                 ! measure hbar^2 / (2 m) in a.u. substitution
!                                 ! r = x^2 in derivative
!                   ! Evaluate numerical derivative of the bra state vector
!                   ! phi(x) / f(x)
!                   z =     cmplx(egfxn(i+1,j,l), kind=fltprec) / cmplx(pathf((x+xh)**2), kind=fltprec)
!                   z = z - cmplx(egfxn(i-1,j,l), kind=fltprec) / cmplx(pathf((x-xh)**2), kind=fltprec)
!                   zt1 = z / cmplx(2.0D0 * xh, kind=fltprec)
!                   z =     cmplx(egfxn(i+1,j,l), kind=fltprec)
!                   z = z - cmplx(egfxn(i-1,j,l), kind=fltprec)
!                   zt2 = z / cmplx(2.0D0 * xh, kind=fltprec)
!                   ! Evaluate numerical derivative of the ket state vector
!                   ! phi(x) / f(x)
!                   z =     cmplx(egfxn(i+1,k,l), kind=fltprec) / cmplx(pathf((x+xh)**2), kind=fltprec)
!                   z = z - cmplx(egfxn(i-1,k,l), kind=fltprec) / cmplx(pathf((x-xh)**2), kind=fltprec)
!                   z = z / cmplx(2.0D0 * xh, kind=fltprec)
!                   zt1 = zt1 * z * cmplx(pf, kind=fltprec)
!                   z =     cmplx(egfxn(i+1,k,l), kind=fltprec)
!                   z = z - cmplx(egfxn(i-1,k,l), kind=fltprec)
!                   z = z / cmplx(2.0D0 * xh, kind=fltprec)
!                   zt2 = zt2 * z * cmplx(pf, kind=fltprec)
!                   zt1 = zt1 - zt2
!                   rework(i) = real(zt1, dblprec)
!                   imwork(i) = real(aimag(zt1), dblprec)
!                end do
!
!                ! Backward difference formula
!                i = ngrd
!                x = xh * real(ngrd, dblprec)
!                pf = real(2.0D0 * x / 2.0D0  / (4.0D0 * x**2), fltprec)     ! Integration
!                                 ! measure hbar^2 / (2 m) in a.u. substitution
!                                 ! r = x^2 in derivative
!                ! Evaluate numerical derivative of the bra state vector
!                ! phi(x) / f(x)
!                z =     cmplx(egfxn(i  ,j,l), kind=fltprec) / cmplx(pathf((x   )**2), kind=fltprec)
!                z = z - cmplx(egfxn(i-1,j,l), kind=fltprec) / cmplx(pathf((x-xh)**2), kind=fltprec)
!                zt1 = z / zh
!                z =     cmplx(egfxn(i  ,j,l), kind=fltprec)
!                z = z - cmplx(egfxn(i-1,j,l), kind=fltprec)
!                zt2 = z / zh
!                ! Evaluate numerical derivative of the ket state vector
!                ! phi(x) / f(x)
!                z =     cmplx(egfxn(i  ,k,l), kind=fltprec) / cmplx(pathf((x   )**2), kind=fltprec)
!                z = z - cmplx(egfxn(i-1,k,l), kind=fltprec) / cmplx(pathf((x-xh)**2), kind=fltprec)
!                z = z / zh
!                zt1 = zt1 * z * cmplx(pf, kind=fltprec)
!                z =     cmplx(egfxn(i  ,k,l), kind=fltprec)
!                z = z - cmplx(egfxn(i-1,k,l), kind=fltprec)
!                z = z / zh
!                zt2 = zt2 * z * cmplx(pf, kind=fltprec)
!                zt1 = zt1 - zt2
!                rework(i) = real(zt1, dblprec)
!                imwork(i) = real(aimag(zt1), dblprec)
!
!                ! Numerical integration
!                re1 = midpointrule (ngrd,xh,rework)
!                im1 = midpointrule (ngrd,xh,imwork)
!                re2 = trapezoidrule(ngrd,xh,rework)
!                im2 = trapezoidrule(ngrd,xh,imwork)
!                r = (re1 - re2)**2 + (im1 - im2)**2
!                if (r > numinterr) then
!                  numinterr = r
!                end if
!                suminterr = suminterr + sqrt(r)
!                fTf(j,k,l) = cmplx(re2, im2, dblprec)
!                fTf(k,j,l) = fTf(j,k,l)                         ! fTf is complex symmetric
!             end do
!          end do
!       end do
!
!       write (*,*) '        Error in numerical integration ', sqrt(numinterr),    &
!      &          '(maximum), ', suminterr / real(nl * nstate * (nstate + 1) / 2,  &
!      &          dblprec), '(mean)'
!       deallocate(rework, imwork)
!       end subroutine numder_fTfmtrx
